;***             ARCL.lsp - Returns the arc data of an arc
;***                                  ARCL.LSP v1.02
;***                    by: Randy Benson        W. R. Benson & Assoc, Inc.
;***                        CIS 75226,1725          13535 Wingo Street
;***                        March, 1993            Arleta, CA 91331-5636
;***
(princ "\nLoading ARCL...")
(defun c:ARCL (/ ang arln brg brgfrm cp cr cs deg degfrm dlta e hftn hlfd
                 lc min mo oldech qtrd rad rd1 rd2 sec sl st twpi xt) 
(defun err (s)
  (if (not (member s '("console break" "Function cancelled")))
    (princ (strcat "\nError: " s))
  )
  (command "UNDO" "E")
  (command "GRAPHSCR")
  (setvar "CMDECHO" oldech)
  (setq *error* olderr)
  (princ)
)
(defun brgfrm ()
  (if (> (strlen brg) 1)
    (progn
      (setq cp (substr brg 1 1)
          brgl (strlen brg)
            cr (substr brg 3 (- brgl 4))
            cs (substr brg (- brgl 1))
      )
    )
    (cond ((or (= "N" brg) (= "S" brg))
            (setq cp brg
                  cr "  0d00'00\""
                  cs " E"
            ))
          ((or (= "E" brg) (= "W" brg))
            (setq cp "N"
                  cr " 90d00'00\""
                  cs (strcat " " brg)
            ))
    )  
  )
)
(defun degfrm (ang)
  (setq deg ""
        min ""
        sec ""
  )
  (while (/= "d" (substr ang 1 1))
    (setq deg (strcat deg (substr ang 1 1))
          ang (substr ang 2)
    )
  )
  (while (< (strlen deg) 3)
    (setq deg (strcat " " deg))
  )
  (setq ang (substr ang 2))
  (while (/= "'" (substr ang 1 1))
    (setq min (strcat min (substr ang 1 1))
          ang (substr ang 2)
    )
  )
  (while (< (strlen min) 2)
    (setq min (strcat "0" min))
  )
  (setq sec (substr ang 2))
  (while (< (strlen sec) 3)
    (setq sec (strcat "0" sec))
  )
  (princ deg)
  (write-char 248)
  (princ (strcat min "'" sec))
  (princ)
)
  (setq olderr *error*
        *error* err
        oldech (getvar "CMDECHO")
  )
  (setvar "CMDECHO" 0)
  (command "UNDO" "G")
(defun etest ()
  (while 
    (null (setq e (entsel "\nPick arc: ")))
    (princ "\010\010\010\010\010\010\010\010\010\010")
    (princ "Missed... Try again:               ")
  )
  (ptest)
)
(defun ptest () 
  (if (not (equal "ARC" (cdr (assoc 0 (entget (car e))))))
    (progn
      (princ "\010\010\010\010\010\010\010\010\010\010")
      (princ "Not an arc... Try again: ")
      (setq e nil)
      (etest)
    )
  )
)
;--------- Begin body of ARCL
  (etest)
  (if (equal 0 (getvar "WORLDUCS"))
    (progn
      (princ "     [ Note: UCS is not world. ]")
      (princ "\n  [ Bearings shown hereon are world coordinate system ]")
    )
  )
  (if e
    (if (= (cdr (assoc 0 (entget (car e)))) "ARC")
      (progn
        (setq twpi (* pi 2.0)
              rad (cdr (assoc 40 (entget (car e))))
              rd1 (cdr (assoc 50 (entget (car e))))
              rd2 (cdr (assoc 51 (entget (car e))))
        )
        (setq dlta (-                                 ;subtract
        rd2                                           ;1st radial (in radians)
        rd1                                           ;from 2nd radial
        ))
        (if (>= dlta twpi)                            ;if too big,
          (setq dlta (- twpi dlta))                   ;subtract 2pi
          (if (< dlta 0)                              ;if too small
            (setq dlta (+ twpi dlta))                 ;add 2pi
          )  
        )
        (setq hlfd (* dlta 0.5)
              qtrd (* dlta 0.25)
        )
        (princ "\n          =====================================\n")
        (princ "                ")
        (princ (car e))
        (princ "\n          =====================================")
        (princ "\n                      Layer = ")
        (princ (cdr (assoc 8 (entget (car e)))))
            (princ "\n                      Color = ")
        (if (not (assoc 62 (entget (car e))))
          (princ "BYLAYER")
          (progn
            (cond
              ((= (cdr (assoc 62 (entget (car e)))) 1) (princ "RED <1>"))
              ((= (cdr (assoc 62 (entget (car e)))) 2) (princ "YELLOW <2>"))
              ((= (cdr (assoc 62 (entget (car e)))) 3) (princ "GREEN <3>"))
              ((= (cdr (assoc 62 (entget (car e)))) 4) (princ "CYAN <4>"))
              ((= (cdr (assoc 62 (entget (car e)))) 5) (princ "BLUE <5>"))
              ((= (cdr (assoc 62 (entget (car e)))) 6) (princ "MAGENTA <6>"))
              ((= (cdr (assoc 62 (entget (car e)))) 7) (princ "WHITE <7>"))
              ((= (cdr (assoc 62 (entget (car e)))) 8) (princ "GRAY <8>"))
              ((= (cdr (assoc 62 (entget (car e)))) 9) (princ "DARK RED <9>"))
              ((= (cdr (assoc 62 (entget (car e)))) 10) (princ "DARK YELLOW <10>"))
              ((= (cdr (assoc 62 (entget (car e)))) 11) (princ "DARK GREEN <11>"))
              ((= (cdr (assoc 62 (entget (car e)))) 12) (princ "DARK CYAN <12>"))
              ((= (cdr (assoc 62 (entget (car e)))) 13) (princ "DARK BLUE <13>"))
              ((= (cdr (assoc 62 (entget (car e)))) 14) (princ "DARK MAGENTA <14>"))
              ((= (cdr (assoc 62 (entget (car e)))) 15) (princ "DARK GRAY <15>"))
              ((> (cdr (assoc 62 (entget (car e)))) 15) (princ "UNDEFINED ( >15 )"))
              (t nil)
            )
          )
        )
        (princ "\n          =====================================")
        (princ "\n                      Delta =      ")
        (setq sl (strlen (rtos rad 2 4)))
        (degfrm (angtos dlta 1 4))
        (princ "\n                     Radius = ")
        (princ (substr (strcat "             " (rtos rad 2 4) "'") sl))
        (princ "\n                 Arc Length = ")
        (setq arln (rtos (* rad dlta) 2 4)
                sl (strlen arln)
        )
        (princ (substr (strcat "             " arln "'") sl))
        (princ "\n                SemiTangent = ")
        (if (<= (abs (- PI dlta)) 0.00000000001) (princ "    <infinite>")
          (progn
            (setq hftn (/ (sin hlfd) (cos hlfd))
                  st (rtos (abs (* hftn rad)) 2 4)
                  sl (strlen st)
            )
            (princ (substr (strcat "             " st "'") sl))
          )
        )
        (princ "\n                 Long Chord = ")  
        (setq lc (* (sin hlfd) (* 2.0 rad))
              sl (strlen (rtos lc 2 4))
        )
        (princ (substr (strcat "             " (rtos lc 2 4) "'") sl))
        (princ "\n            Middle Ordinate = ")  
        (setq mo (rtos (* (- 1 (cos hlfd)) rad) 2 4)
              sl (strlen mo)
        )
        (princ (substr (strcat "             " mo "'") sl))
        (princ "\n                   External = ")  
        (if (<= (abs (- PI dlta)) 0.00000000001) (princ "    <infinite>")
          (progn
            (setq xt (rtos (abs (* hftn rad (/ (sin qtrd) (cos qtrd)))) 2 4)
                  sl (strlen xt)
            )
            (princ (substr (strcat "             " xt "'") sl))
          )
        )
        (princ "\n          =====================================")
        (princ "\n              Chord Bearing =     ")
        (setq brg (angtos (+ (/ pi 2.0) (/ (+ rd1 rd2) 2.0)) 4 4))
        (brgfrm)
        (princ cp)
        (degfrm cr)
        (princ cs)
        (princ "\n          =====================================")
        (princ "\n           Radial Bearing 1 =     ")
        (setq brg (angtos rd1 4 4))
        (brgfrm)
        (princ cp)
        (degfrm cr)
        (princ cs)
        (princ "\n           Radial Bearing 2 =     ")
        (setq brg (angtos rd2 4 4))
        (brgfrm)
        (princ cp)
        (degfrm cr)
        (princ cs)
        (princ "\n          =====================================")
        (princ "\n          Tangent Bearing 1 =     ")
        (setq brg (angtos (+ (/ pi 2) rd1) 4 4))
        (brgfrm)
        (princ cp)
        (degfrm cr)
        (princ cs)
        (princ "\n          Tangent Bearing 2 =     ")
        (setq brg (angtos (+ (/ pi 2.0) rd2) 4 4))
        (brgfrm)
        (princ cp)
        (degfrm cr)
        (princ cs)
        (princ "\n          =====================================")
        (textscr)
        (getpoint "\nPress <Enter> to continue... ") 
        (princ)  
        (graphscr)
      )
    )
  )  
  (setvar "CMDECHO" oldech)
  (setq   *error*   olderr
              err   nil)
  (princ)
)
(princ " Loaded.")
(princ)
